package question3;

import java.util.LinkedList;
import java.util.Queue;

public class BinaryTree {

  /**
   * Display a binary tree in string format.
   * @param root of a binary tree.
   * @return string representation of the tree.
   */
  public static String toString(Node root) {
    StringBuilder sb = new StringBuilder();
    sb.append("[");
    Queue<Node> queue = new LinkedList<>();
    queue.add(root);
    while (!queue.isEmpty()) {
      Node n = queue.remove();
      if (n != null) {
        queue.add(n.left);
        queue.add(n.right);
      }
      sb.append(n + ", ");
    }
    sb.setLength(sb.length() - 2);
    sb.append("]");
    return sb.toString();
  }

  /**
   * Convert a rank array a linked representation of a binary tree.
   * @param levelOrder rank array representation of the binary tree.
   *        Pre: ranked array represents a valid binary tree.
   * @return linked representation of a binary tree.
   */
  public static Node toTree(Integer[] levelOrder) {
    int n = levelOrder.length;
    Node root = new Node(levelOrder[1]);
    Node[] node = new Node[n];
    node[1] = root;
    Integer value;
    for (int i = 1; i < n / 2; i++) {
      if (node[i] == null) {
        continue;
      }
      value = levelOrder[2 * i];
      node[i].left = value == null ? null : new Node(value);
      value = levelOrder[2 * i + 1];
      node[i].right = value == null ? null : new Node(value);
      node[2 * i] = node[i].left;
      node[2 * i + 1] = node[i].right;
    }
    return root;
  }

  /**
   * Demo.
   * @param args not used here.
   */
  public static void main(String[] args) {
    Integer[] sample1 = {null, 4, 2, 5, null, 3};
    Node root1 = toTree(sample1);
    System.out.println(toString(root1));

    Integer[] sample2 = {null, 20, 18, 21, 16, null};
    Node root2 = toTree(sample2);
    System.out.println(toString(root2));
  }
}
